﻿/*
* Functions
*/
function getVersion() {
    var version = 'NaN';
    var xhr = new XMLHttpRequest();
    xhr.open('GET', chrome.extension.getURL('manifest.json'), false);
    xhr.send(null);
    var manifest = JSON.parse(xhr.responseText);
    return manifest.version;
}

function startsWith(str1, str2) {
	return (str1.substring(0, str2.length) === str2);
}

function roundNumber(num, dec) {
	var result = Math.round(num*Math.pow(10,dec))/Math.pow(10,dec);
	return result;
}

function bytesToSize(bytes) {
    var sizes = ['B', 'KB', 'MB', 'GB', 'TB'];
    if (bytes == 0) return 'n/a';
    var i = parseInt(Math.floor(Math.log(bytes) / Math.log(1024)));
    return roundNumber(bytes / Math.pow(1024, i), 2) + ' ' + sizes[i];
};

function getLocalTime(d) {
	var mins = d.getMinutes() + "";
	if (mins.length == 1) {
		mins = "0" + mins;
	}
	var lu = '' + d.getDate() + '/' + (d.getMonth()+1) + '/' + d.getFullYear() + ' ' + d.getHours() + ':' + mins;
	return lu;
}

function escapeHTML(str) {
    return str.replace(/&/g,'&amp;').replace(/</g,'&lt;').replace(/>/g,'&gt;');
}

function shortenUrl(url) {
	var decodedURL = url;
	try {
		decodedURL = decodeURI(decodedURL);
	} catch(err) {
		decodedURL = url;
	}
	if (decodedURL.length > 50) {
		decodedURL = decodedURL.substring(0, 40) + ' ... ' + decodedURL.substring(decodedURL.length-10);
	}
	return escapeHTML(decodedURL);
}

/*
* Normal distribution
*/
function normal() {
	var u1,u2;
	u1 = u2 = 0.;
	while (u1 * u2 == 0.) {
		u1 = Math.random();
		u2 = Math.random();
	}
	return Math.sqrt(-2. * Math.log(u1)) * Math.cos(2 * Math.PI * u2);
}

function gauss(mean,stdev) {
	return stdev * normal() + 1. * mean;
}

/*
* Storage
*/
function sortLogs(logs) {
	logs.sort(function(x, y) {
		return (y.date-x.date);
	});
	return logs;
}

function isValidJson(str) {
	try {
		var jsonObject = JSON.parse(str);
	} catch(e) {
		return false;
	}
	return true;
}

function purgeTable(name) {
	var span = 1000*60*60*24;
	var now = (new Date()).getTime();
	
	var logs = JSON.parse(localStorage[name]);
	var newLogs = [];
	logs.forEach(function(element, index, array) {
		if (now - element.date < span) {
			newLogs.push(element);
		}
	});
	localStorage[name] = JSON.stringify(newLogs);
}

function purge() {
	purgeTable("errors");
	purgeTable("submissions");
}

function createSchema() {
	if (!isValidJson(localStorage["options"])) {
		localStorage["options"] = "{}";
	}
	if (!isValidJson(localStorage["errors"])) {
		localStorage["errors"] = "[]";
	}
	if (!isValidJson(localStorage["submissions"])) {
		localStorage["submissions"] = "[]";
	}
	purge();
}

function getDefaultOption(key) {
	if (key == "refetch") {
		return 15;
	} else if (key == "refetch_var") {
		return 3;
	} else {
		return undefined;
	}
}

function getOption(key) {
	createSchema();
	var tmp = JSON.parse(localStorage["options"]);
	if (!tmp[key]) {
		var def = getDefaultOption(key);
		setOption(key, def);
		return def;
	}
	return tmp[key];
}

function setOption(key, val) {
	createSchema();
	var options = JSON.parse(localStorage["options"]);
	options[key] = val;
	localStorage["options"] = JSON.stringify(options);
}

function insertError(url,text,error) {
	createSchema();
	var now = (new Date()).getTime();
	var errors = JSON.parse(localStorage["errors"]);
	errors.push({'date': now, 'url': url, 'text': text, 'error': error});
	localStorage["errors"] = JSON.stringify(errors);
}

function getErrors() {
	createSchema();
	return sortLogs(JSON.parse(localStorage["errors"]));
}

function insertSubmission(query, urls) {
	createSchema();
	var now = (new Date()).getTime();
	var submissions = JSON.parse(localStorage["submissions"]);
	submissions.push({'date': now, 'query': query, 'results': urls});
	localStorage["submissions"] = JSON.stringify(submissions);
}

function getSubmissions() {
	createSchema();
	return sortLogs(JSON.parse(localStorage["submissions"]));
}

/*
* parseUri 1.2.2
*/
function parseUri(str) {
	var	o   = parseUri.options,
		m   = o.parser[o.strictMode ? "strict" : "loose"].exec(str),
		uri = {},
		i   = 14;

	while (i--) uri[o.key[i]] = m[i] || "";

	uri[o.q.name] = {};
	uri[o.key[12]].replace(o.q.parser, function ($0, $1, $2) {
		if ($1) uri[o.q.name][$1] = $2;
	});

	return uri;
};

parseUri.options = {
	strictMode: false,
	key: ["source","protocol","authority","userInfo","user","password","host","port","relative","path","directory","file","query","anchor"],
	q:   {
		name:   "queryKey",
		parser: /(?:^|&)([^&=]*)=?([^&]*)/g
	},
	parser: {
		strict: /^(?:([^:\/?#]+):)?(?:\/\/((?:(([^:@]*)(?::([^:@]*))?)?@)?([^:\/?#]*)(?::(\d*))?))?((((?:[^?#\/]*\/)*)([^?#]*))(?:\?([^#]*))?(?:#(.*))?)/,
		loose:  /^(?:(?![^:@]+:[^:@\/]*@)([^:\/?#.]+):)?(?:\/\/)?((?:(([^:@]*)(?::([^:@]*))?)?@)?([^:\/?#]*)(?::(\d*))?)(((\/(?:[^?#](?![^?#\/]*\.[^?#\/.]+(?:[?#]|$)))*\/?)?([^?#\/]*))(?:\?([^#]*))?(?:#(.*))?)/
	}
};

/*
 * Register error callback
 */
window.onerror = function(message, url, lineNumber) {
	insertError(url,lineNumber,message);
	return true;
};